<?php
/**
 * The resultbygraph view file of thinkrun module of ZenTaoPMS.
 * @copyright   Copyright 2009-2023 禅道软件（青岛）有限公司(ZenTao Software (Qingdao) Co., Ltd. www.zentao.net)
 * @license     ZPL(https://zpl.pub/page/zplv12.html) or AGPL(https://www.gnu.org/licenses/agpl-3.0.en.html)
 * @author      Yu Zhang <zhangyu@easycorp.ltd>
 * @package     thinkrun
 * @link        https://www.zentao.net
 */
namespace zin;

$bodyTrs    = array();
$stepIndex  = 1;
$objectHead = array();
$objectBody = array();

function renderTableInput($stepOptions, $stepAnswer)
{
    global $lang;
    $result            = (array) $stepAnswer->result;
    $customFields      = !empty($stepAnswer->customFields) ? get_object_vars($stepAnswer->customFields) : array();
    $newFileds         = array_merge($stepOptions->fields, $customFields);
    $tableInputBodyTrs = [];
    $inputUnit         = '';

    foreach($newFileds as $index => $item)
    {
        if(!empty($stepOptions->inputType) && $stepOptions->inputType == '1') $inputUnit = ' %';
        if(!empty($stepOptions->inputType) && $stepOptions->inputType == '2') $inputUnit = ' ' . $lang->thinkrun->amountUnit[$stepAnswer->amountUnit[$index]];
        $tableInputBodyTrs[] = h::tr
        (
            h::td(setClass('w-1/5'), setStyle(array('font-weight' => 'bold')), $item),
            h::td(isset($result[$index]) ? span($result[$index] . ($stepOptions->questionType == 'score' ? ' ' . $lang->thinkrun->tableResult->points : '') . $inputUnit) : span($lang->thinkrun->tableResult->notFilledIn))
        );
    }
    return h::table
    (
        setClass('w-full'),
        h::tbody($tableInputBodyTrs)
    );
}

function renderMulticolumn($stepFields, $result)
{
    global $lang;
    $multicolumnHead    = array();
    $multicolumnBodyTrs = array();
    $maxResultCount     = count((array) reset($result));
    $emptyResult        = true;

    foreach($stepFields as $fieldIndex => $filed)
    {
        $multicolumnHead[] = h::th(setStyle(array('font-weight' => 'bold')), $filed);
    }
    for($i = 1; $i <= $maxResultCount; $i++)
    {
        $row = array();
        $emptyCount = 0;
        foreach($stepFields as $fieldIndex => $filed)
        {
            if(!empty($result['col' . ($fieldIndex + 1)]))
            {
                $columnResult = $result['col' . ($fieldIndex + 1)]->$i;
                $row[] = h::td(!empty($columnResult) ? $columnResult : span(setClass('hidden'), $lang->thinkrun->tableResult->notFilledIn));
                if(empty($columnResult)) $emptyCount ++;
            }
        }
        $multicolumnBodyTrs[] = $emptyCount >= count($stepFields) ? null : h::tr($row);
    }
    foreach($multicolumnBodyTrs as $multicolumnBodyItem)
    {
        if(!empty($multicolumnBodyItem)) $emptyResult = false;
    }
    return h::table
    (
        setClass('w-full'),
        h::thead(setClass(!empty($emptyResult) ? 'empty-multicolumn' : ''), $multicolumnHead),
        empty($emptyResult) ? h::tbody($multicolumnBodyTrs) : null
    );
}

function fnBuildCheckboxTable($blockIndex, $stepLink, $stepOptions, $stepAnswer)
{
    global $lang;
    $result = (array) $stepAnswer->result;
    $answer = array();
    if(!empty($stepLink->unSelectedBlock) && $stepLink->unSelectedBlock == $blockIndex)
    {
        $unSelectedItems = array_diff($stepOptions->fields, $result);
        foreach($unSelectedItems as $unSelectedItem)
        {
            $answer[] = span(setClass('mr-3'), $unSelectedItem);
        }
        if(isset($stepOptions->enableOther) && $stepOptions->enableOther === 'on' && empty($stepAnswer->other)) $answer[] = $lang->thinkrun->other;
    }
    else
    {
        foreach($result as $resultItem)
        {
            $answer[] = span(setClass('mr-3'), $resultItem === 'other' ? $stepAnswer->other : $resultItem);
        }
        if(empty($answer)) $answer[] = span($lang->thinkrun->tableResult->notFilledIn);
    }
    return $answer;
}

$fnBuildBlockTable = function(array $steps, string $type='block', int $blockIndex=0, string $block='', bool $showDefaltBlock=false, string $nodeTitle='')
{
    global $lang, $config;
    $fnBlockTable = array();
    $letter = range('a', 'z');
    if(empty($steps) && $type == 'block')
    {
        $fnBlockTable[] = h::tr
        (
            h::td($block . ($showDefaltBlock ? ' (' . $letter[$blockIndex - 1] . $lang->thinkrun->tableResult->block . ')' : '')),
            h::td(set::colspan(3), $lang->thinkrun->tableResult->noneContent)
        );
        return $fnBlockTable;
    }

    foreach($steps as $stepBlockIndex => $step)
    {
        $stepLink     = !empty($step->link) ? json_decode($step->link) : new stdClass();
        $stepOptions  = $step->options;
        $questionType = $stepOptions->questionType;
        $stepAnswer   = $step->answer;
        $result       = (array) $stepAnswer->result;
        $answer       = array();

        if($stepOptions->questionType === 'multicolumn')
        {
            $answer = renderMulticolumn($stepOptions->fields, $result);
        }
        elseif($questionType === 'radio' || $questionType === 'checkbox')
        {
            $answer = fnBuildCheckboxTable($blockIndex, $stepLink, $stepOptions, $stepAnswer);
        }
        elseif($stepOptions->questionType === 'tableInput' || $questionType == 'score')
        {
            $answer = renderTableInput($stepOptions, $stepAnswer);
        }
        else
        {
            $answer = !empty($result) ? $result : $lang->thinkrun->tableResult->notFilledIn;
        }
        $colunm1     = '';
        $scoreSystem = '';
        if($type == 'block') $colunm1 = $block . ($showDefaltBlock ? ' (' . $letter[$blockIndex - 1] . $lang->thinkrun->tableResult->block . ')' : '');
        if($type == 'node')  $colunm1 = $nodeTitle;
        if($questionType == 'score')
        {
            $scoreSystem = '（' . $lang->thinkwizard->questions->scoreList[$stepOptions->scoreSetting] . '）';
        }
        $fnBlockTable[] = h::tr
        (
            $stepBlockIndex == '0' ? h::td($colunm1, set::rowspan(count($steps))) : null,
            h::td($lang->thinkrun->tableResult->tableQuestionType[$questionType]),
            h::td($step->title . $scoreSystem),
            h::td(setClass(in_array($questionType, $config->thinkrun->resultTableType) ? 'multicolumn-table' : ''), $answer)
        );
    }
    return $fnBlockTable;
};

if(in_array($wizard->model, $this->config->thinkwizard->venn))
{
    foreach($wizard->blocks as $index => $block)
    {
        $steps     = $this->thinkrunZen->buildBlockTableSteps($questionSteps, $index);
        $bodyTrs[] = $fnBuildBlockTable($steps, 'block', $index + 1, $block, $wizard->model == '3c');
    }
}
elseif(in_array($wizard->model, $this->config->thinkwizard->hiddenMenuModel))
{
    $stepTreeIndex = 0;
    foreach($stepsTree as $stepTreeItem)
    {
        $steps     = array_filter($stepTreeItem->children, function($item) {return $item->type == 'question';});
        $bodyTrs[] = $fnBuildBlockTable($steps, 'node', 0, '', false, $stepTreeItem->title);
        $stepTreeIndex ++;
    }
    if($wizard->type == 'RDA')
    {
        foreach($objectsResult['objectHead'] as $objectHeadItem){$objectHead[] = h::th(div($objectHeadItem));}
        foreach($objectsResult['objectBody'] as $objectBodyItems)
        {
            $fnTableTd = array();
            foreach ($objectBodyItems as $objectTitle => $objectBodyItem){$fnTableTd[] = h::td($objectBodyItem);}
            $objectBody[] = h::tr($fnTableTd);
        }
    }
    if($wizard->type == 'MQA')
    {
        $amountTitle = array('lastValue', 'currentValue', 'marketValue', 'competitorsValue');
        foreach($objectsResult['objectHead'] as $objectHeadItem){$objectHead[] = h::th(div($objectHeadItem));}
        foreach($objectsResult['objectBody'] as $objectBodyItems)
        {
            $fnTableTd = array();
            foreach ($objectBodyItems as $objectTitle => $objectBodyItem){$fnTableTd[] = h::td(setClass(in_array($objectTitle, $amountTitle) ? 'amount-body' : ''), $objectBodyItem);}
            $objectBody[] = h::tr($fnTableTd);
        }
    }
}
else
{
    foreach($questionSteps as $step)
    {
        $stepOptions   = $step->options;
        $questionType  = $stepOptions->questionType;
        $stepAnswer    = $step->answer;
        $result        = (array) $stepAnswer->result;
        $answer        = array();
        $tableInputHrs = array();

        if($questionType === 'tableInput' || $questionType == 'score')
        {
            $answer[] = renderTableInput($stepOptions, $stepAnswer);
        }
        elseif($questionType == 'multicolumn')
        {
            $answer[] = renderMulticolumn($stepOptions->fields, $result);
        }
        else
        {
            foreach($result as $item)
            {
                $answer[] = span(setClass('mr-3'), $item === 'other' ? $stepAnswer->other : $item);
            }
            if(empty($answer)) $answer[] = span($lang->thinkrun->tableResult->notFilledIn);
        }
        $bodyTrs[] = h::tr
        (
            h::td($stepIndex),
            h::td($lang->thinkrun->tableResult->tableQuestionType[$questionType]),
            h::td($step->title),
            h::td(setClass(in_array($questionType, $config->thinkrun->resultTableType) ? 'multicolumn-table' : ''), $answer)
        );
        $stepIndex ++;
    }
}

$head1 = $lang->thinkrun->tableResult->number;
if(in_array($wizard->model, $this->config->thinkwizard->venn)) $head1 = $lang->thinkrun->tableResult->blockName;
if(in_array($wizard->model, $this->config->thinkwizard->hiddenMenuModel)) $head1 = $lang->thinkrun->tableResult->node;

panel
(
    setStyle(array('overflow' => 'unset')),
    set::bodyClass('p-4'),
    setID('runResult'),
    div
    (
        setClass('leading-normal w-full text-left text-xl wizard-introduction'),
        $wizard->introduction
    ),
    in_array($wizard->model, $this->config->thinkwizard->hiddenMenuModel) ? ul
    (
        setClass('regionMenu mt-3'),
        li(set::className('text-md', $tab == 'question' ? ' active' : ''), a(set::href(inlink('result', "marketID=$marketID&runID=$run->id&browseType=$browseType&type=$type&tab=question")), span($lang->thinkrun->tableResult->viewByProblem))),
        li
        (
            setClass('text-md', $tab == 'object' ? ' active' : ''),
            a(set::href(inlink('result', "marketID=$marketID&runID=$run->id&browseType=$browseType&type=$type&tab=object")), span($lang->thinkrun->tableResult->viewByObject))
        )
    ) : null,
    div
    (
        setClass('result-table overflow-auto question-table'),
        $wizard->model == '3c' ? h::table
        (
            setID('thinkrun-result-table'),
            setClass('w-full border-primary border-opacity-50 border text-base font-normal text-gray-700'),
            h::thead
            (
                setClass('sticky table-head'),
                setStyle(array('top' => '-1px')),
                h::tr
                (
                    setClass('bg-primary-50 h-9'),
                    h::th(setStyle('text-align', 'center'), $lang->thinkrun->tableResult->resultsGraphical)
                )
            ),
            h::tbody
            (
                h::tr
                (
                    h::td(img(setClass('w-1/4 m-auto p-2'), set::src('data/thinmory/background/block3c.svg')))
                )
            )
        ) : null,
        h::table
        (
            setID('thinkrun-result-table'),
            setClass('w-full border-primary border-opacity-50 border text-base font-normal text-gray-700', $tab != 'question' ? ' hidden' : ''),
            h::thead
            (
                setClass('sticky table-head'),
                setStyle(array('top' => '-1px')),
                h::tr
                (
                    setClass('bg-primary-50 h-9 table-header'),
                    h::th(setClass('w-20 min-number'), div($head1)),
                    h::th(setClass('w-28 min-type'), div($lang->thinkrun->tableResult->questionType)),
                    h::th(setClass('w-1/3 min-name'), div($lang->thinkrun->tableResult->questionName)),
                    h::th(div(in_array($wizard->model, $this->config->thinkwizard->venn) || in_array($wizard->model, $this->config->thinkwizard->hiddenMenuModel) ? $lang->thinkrun->tableResult->exhibitContent : $lang->thinkrun->tableResult->answer))
                )
            ),
            h::tbody($bodyTrs)
        )
    ),
    div
    (
        setClass('result-table overflow-auto object-table', $tab != 'object' ? ' hidden' : ''),
        h::table
        (
            setID('thinkrun-result-table'),
            setClass('w-full border-primary border-opacity-50 border text-base font-normal text-gray-700'),
            h::thead(setClass('sticky table-head'), setStyle(array('top' => '-1px')), h::tr(setClass('bg-primary-50 h-9 table-header'), $objectHead)),
            h::tbody($objectBody)
        )
    ),
    !empty($wizard->suggestion) ? div
    (
        setClass('w-full leading-5 text-base mt-4 py-4.5 text-left wizard-suggestion'),
        section
        (
            setClass('break-words'),
            set::content( htmlspecialchars_decode($wizard->suggestion)),
            set::useHtml(true)
        )
    ) : null,
    $run->conclusion ? div(
        setClass('w-full leading-5 text-base mt-4 py-4.5 text-left run-conclusion'),
        $run->conclusion
    ) : null
);
